/*
 * Project name:
     HID_Read_Write_Interrupt (Testing the USB HID connection)
     -> modified in simple HID keyboard

 * Test configuration:
     MCU:             PIC18F47J53
                      http://ww1.microchip.com/downloads/en/DeviceDoc/39964B.pdf
     Dev.board:       PIC clicker - ac:PIC_clicker
                      http://www.mikroe.com/pic/clicker/
     Oscillator:      HS-PLL 48.0000 MHz, 16.0000 MHz Crystal
     Ext. Modules:    None.
     SW:              mikroC PRO for PIC
                      http://www.mikroe.com/mikroc/pic/

 * Comments:
     Here is some modification of original MikroE example
     in order to work as HID Keyboard. Enjoy!
   
     Note that this simple code recognize only a pression at once,
     if you hold the button, it dont care.
 
 * Date: 23/5/2014

 */

// configuration for Pic Clicker HW
sbit LD1 at LATA0_bit;
sbit LD2 at LATA1_bit;
sbit LD1_Direction at TRISA0_bit;
sbit LD2_Direction at TRISA1_bit;
sbit T1 at RD3_bit;
sbit T2 at RD2_bit;
sbit T1_Direction at TRISD3_bit;
sbit T2_Direction at TRISD2_bit;                       

unsigned char readbuff[64] absolute 0x500;             // buffer USB
unsigned char writebuff[64] absolute 0x540;

unsigned char modifier=0b00000000;
unsigned char rsv=0;
unsigned char keycode[6];

char cnt = 0x04;
char oldstate1 = 0;
char oldstate2 = 0;

void interrupt(){
   USB_Interrupt_Proc();                   // USB servicing is done inside the interrupt
}

void reset_keycode()
{
  modifier=0;
  keycode[0]=0;             // initializing keycode
  keycode[1]=0;
  keycode[2]=0;
  keycode[3]=0;
  keycode[4]=0;
  keycode[5]=0;
  writebuff[0]=0x00;                            // reset buffer
  writebuff[1]=rsv;
  writebuff[2]=keycode[0];
  writebuff[3]=keycode[1];
  writebuff[4]=keycode[2];
  writebuff[5]=keycode[3];
  writebuff[6]=keycode[4];
  writebuff[7]=keycode[5];
}

void send_key(unsigned char key, unsigned char mod)
{
  keycode[0] = key;

  writebuff[0]=mod;                             // build the message
  writebuff[1]=rsv;                             // and write into buffer
  writebuff[2]=keycode[0];
  writebuff[3]=keycode[1];
  writebuff[4]=keycode[2];
  writebuff[5]=keycode[3];
  writebuff[6]=keycode[4];
  writebuff[7]=keycode[5];
  
  while(!HID_Write(&writebuff,8));              // write HID command
  delay_ms(30);
  
  reset_keycode();                              // button release message
  while(!HID_Write(&writebuff,8));
  delay_ms(30);
  
}

void main()
{
  ANCON0 = 0;
  ANCON1 = 0;
  T1_Direction = 1;         // Set direction for buttons
  T2_Direction = 1;
  LD1_Direction = 0;        // Set direction for LEDS
  LD2_Direction = 0;
  LD1 = 0;                  // turn off leds
  LD2 = 0;

  HID_Enable(&readbuff, &writebuff);       // Enable HID communication
  GIE_bit = 1;
  //LD1 = 1;                                 // turn on led
  
  while(1)
  {
    reset_keycode();
    
    // check T1 button
    if (Button(&PORTD, 3, 2, 0)) {                // Detect logical zero
      oldstate1 = 1;                              // Update flag
      LD1 = 1;
    }
    if (oldstate1 && Button(&PORTD, 3, 2, 1)) {   // Detect zero-to-one transition
      oldstate1 = 0;                              // Update flag
      send_key(0x19,0x01);                        // send CTRL+V
      LD1 ^= 1;
    }
    
    // check T2 button
    if (Button(&PORTD, 2, 2, 0)) {                // Detect logical zero
      oldstate2 = 1;                              // Update flag
      LD2 = 1;
    }
    if (oldstate2 && Button(&PORTD, 2, 2, 1)) {   // Detect zero-to-one transition
      oldstate2 = 0;                              // Update flag
      send_key(0x06, 0x01);                       // send CTRL+C
      LD2 ^= 1;
    }
  }
}